﻿using ICSharpCode.SharpZipLib.Zip;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;

namespace MxB
{
    internal class Program
    {
        public class RootJobs
        {
            public bool enabled { get; set; }
            public string name { get; set; }
            public List<string> source { get; set; }
            public List<string> destination { get; set; }
            public bool zip { get; set; }
            public bool inc { get; set; }
        }

        public static string version = "1.5.0";
        public static string zipname = "";
        public static string zippath = "";
        static void Main(string[] args)
        {
            string month = ZeroPad(DateTime.Now.Month);
            string day = ZeroPad(DateTime.Now.Day);
            string year = ZeroPad(DateTime.Now.Year);
            string hour = ZeroPad(DateTime.Now.Hour);
            string minute = ZeroPad(DateTime.Now.Minute);

            var lp = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);

            Utilities.Message.Splash();

            if (args.Length == 0)
            {
                var dirl = Directory.EnumerateFiles(@"jobs", "*.json");
                foreach (var f in dirl)
                {
                    RootJobs j = JsonConvert.DeserializeObject<RootJobs>(File.ReadAllText($@"{f}"));
                    Utilities.Message.CMW($"[{j.name}] Processing...", true, 1);
                    if (j.enabled)
                    {
                        int sc = j.source.Count;
                        int scur = 1;
                        foreach (var js in j.source)
                        {
                            var sourceType = js.Split('|')[0];
                            var sourceName = js.Split('|')[1];
                            var sourcePath = js.Split('|')[2];
                            DirectoryInfo dInfo = new DirectoryInfo(sourcePath);
                            long sizeOfDir = DirectorySize(dInfo, true);
                            Utilities.Message.CMW($"[{j.name}][{sourceType}][{((double)sizeOfDir) / (1024 * 1024):N2}MB]{scur}/{sc}] Processing...", true, 1);                            
                            if (!Directory.Exists($@"tmp\{j.name}")) { Directory.CreateDirectory($@"tmp\{j.name}"); }
                            if (!Directory.Exists($@"tmp\{j.name}\{sourceName}")) { Directory.CreateDirectory($@"tmp\{j.name}\{sourceName}"); }
                            if (args.Contains("--exclude")) {
                                CopyFilesRecursively(sourcePath, $@"tmp\{j.name}\{sourceName}", args);
                            } else
                            {
                                CopyFilesRecursively(sourcePath, $@"tmp\{j.name}\{sourceName}", null);
                            }                           
                            scur++;
                        }
                        foreach (var dest in j.destination)
                        {
                            if (!Directory.Exists($@"{dest}\{j.name}")) { Directory.CreateDirectory($@"{dest}\{j.name}"); }
                            if (j.zip)
                            {                                
                                CreateZip(j.name, day, hour, lp);
                                File.Copy($@"tmp\{j.name}-D{day}-H{hour}.zip", $@"{dest}\{j.name}\{j.name}-D{day}-H{hour}-M{minute}.zip");
                                ClearFolder($@"tmp");
                            }
                            else
                            {
                                CopyFilesRecursively($@"tmp\{j.name}", dest, null);
                            }
                        }
                    }
                    else
                    {
                        Utilities.Message.CMW($"[{j.name}] Disabled > Skipping.", true, 1);
                    }
                }
            } else if (args.Contains("--job"))
            {
                string jnarg = args[1];
                RootJobs j = JsonConvert.DeserializeObject<RootJobs>(File.ReadAllText($@"{lp}\jobs\{jnarg}.json"));
                Utilities.Message.CMW($"[{j.name}] Processing...", true, 1);
                if (j.enabled)
                {
                    int sc = j.source.Count;
                    int scur = 1;
                    foreach (var js in j.source)
                    {
                        var sourceType = js.Split('|')[0];
                        var sourceName = js.Split('|')[1];
                        var sourcePath = js.Split('|')[2];
                        long sizeOfDir = 0;
                        if (sourceType == "D") { DirectoryInfo dInfo = new DirectoryInfo(sourcePath); sizeOfDir = DirectorySize(dInfo, true); }
                        if (sourceType == "F") { FileInfo fInfo = new FileInfo(sourcePath); sizeOfDir = FileSize(fInfo); }

                        Utilities.Message.CMW($"[{j.name}][{sourceType}][{((double)sizeOfDir) / (1024 * 1024):N2}MB]{scur}/{sc}] Processing...", true, 1);
                        if (!Directory.Exists($@"tmp\{j.name}")) { Directory.CreateDirectory($@"tmp\{j.name}"); }                                           
                        switch (sourceType)
                        {
                            case "D":
                                if (!Directory.Exists($@"tmp\{j.name}\{sourceName}")) { Directory.CreateDirectory($@"tmp\{j.name}\{sourceName}"); }
                                if (args.Contains("--exclude"))
                                {
                                    CopyFilesRecursively(sourcePath, $@"tmp\{j.name}\{sourceName}", args);
                                }
                                else
                                {
                                    CopyFilesRecursively(sourcePath, $@"tmp\{j.name}\{sourceName}", null);
                                }
                                break;
                            case "F":
                                if (!Directory.Exists($@"tmp\{j.name}\{sourceName}")) { Directory.CreateDirectory($@"tmp\{j.name}\{sourceName}"); }
                                string fn = Path.GetFileName(sourcePath);
                                File.Copy(sourcePath, $@"tmp\{j.name}\{sourceName}\{fn}");
                                break;
                        }
                        scur++;
                    }
                    foreach (var dest in j.destination)
                    {
                        if (!Directory.Exists($@"{dest}\{j.name}")) { Directory.CreateDirectory($@"{dest}\{j.name}"); }
                        if (j.zip)
                        {
                            CreateZip(j.name, day, hour, lp);
                            File.Copy($@"tmp\{j.name}-D{day}-H{hour}.zip", $@"{dest}\{j.name}\{j.name}-D{day}-H{hour}-M{minute}.zip");
                            ClearFolder($@"tmp");
                        }
                        else
                        {
                            CopyFilesRecursively($@"tmp\{j.name}", dest, null);
                        }
                    }
                }
                else
                {
                    Utilities.Message.CMW($"[{j.name}] Disabled > Skipping.", true, 1);
                }
            } else if (args.Contains("--clean"))
            {
                int cc = Convert.ToInt32(args[2]);
                string jnarg = args[1];
                RootJobs j = JsonConvert.DeserializeObject<RootJobs>(File.ReadAllText($@"{lp}\jobs\{jnarg}.json"));
                Utilities.Message.CMW($"[{j.name}] Cleaning...", true, 1);
                foreach (var d in j.destination)
                {
                    var directory = new DirectoryInfo(d);
                    var query = directory.GetFiles("*.zip", SearchOption.AllDirectories);
                    foreach (var file in query.OrderBy(file => file.CreationTime).Skip(cc))
                    {
                        file.Delete();
                    }
                }
            }         
        }

        static long FileSize(FileInfo fInfo)
        {
            // Enumerate all the files
            long totalSize = fInfo.Length;
            
            return totalSize;

        }

        static long DirectorySize(DirectoryInfo dInfo, bool includeSubDir)
        {
            // Enumerate all the files
            long totalSize = dInfo.EnumerateFiles()
                         .Sum(file => file.Length);

            // If Subdirectories are to be included
            if (includeSubDir)
            {
                // Enumerate all sub-directories
                totalSize += dInfo.EnumerateDirectories()
                         .Sum(dir => DirectorySize(dir, true));
            }
            return totalSize;

        }

        private static void CopyFilesRecursively(string sourcePath, string targetPath, string[] exclude)
        {
            //Now Create all of the directories
            foreach (string dirPath in Directory.GetDirectories(sourcePath, "*", SearchOption.AllDirectories))
            {
                Directory.CreateDirectory(dirPath.Replace(sourcePath, targetPath));
            }

            //Copy all the files & Replaces any files with the same name
            foreach (string newPath in Directory.GetFiles(sourcePath, "*.*", SearchOption.AllDirectories))
            {
                var ex = Path.GetExtension(newPath);
                if (exclude != null)
                {
                    string[] ext = exclude[3].Split(',').ToArray();
                    if (!ext.Contains(ex))
                    {
                        File.Copy(newPath, newPath.Replace(sourcePath, targetPath), true);
                    }
                } else
                {
                    File.Copy(newPath, newPath.Replace(sourcePath, targetPath), true);
                }                          
            }
        }

        private static void ClearFolder(string FolderName)
        {
            DirectoryInfo dir = new DirectoryInfo(FolderName);

            foreach (FileInfo fi in dir.GetFiles())
            {
                fi.Delete();
            }

            foreach (DirectoryInfo di in dir.GetDirectories())
            {
                ClearFolder(di.FullName);
                di.Delete();
            }
        }

        public static void CreateZip(string bn, string day, string hour, string lp)
        {
            // Compress name in tmp dir.
            FastZip fz = new FastZip();
            zipname = $@"{bn}-D{day}-H{hour}.zip";
            zippath = $@"{lp}\tmp\{zipname}";
            fz.CreateZip(zippath, $@"{lp}\tmp\{bn}", true, "");
        }

        public static string ZeroPad(int src)
        {
            if (src < 10)
            {
                return $"0{src.ToString()}";
            }
            else
            {
                return src.ToString();
            }
        }
    }
}
